#!/bin/bash
# Copyright 2016 Michal Žejdl
# APC UPS - RunTimeRemaining, LineVoltage

# at least five args are required
test $# -lt 5 && exit

rrdbasedir=$1
rrdfile=$2
outputdir=$3
starttime=$4
endtime=$5

# make the remote flush possible
cd $rrdbasedir || exit

test -d "$outputdir" || mkdir -p "$outputdir" || exit

# lazy (-z) is handy
options="-s $starttime -e $endtime -z"

# append rrdcached
test $# -gt 5 && options="$options --daemon $6"

filename=$(basename $rrdfile .rrd)

rrdtool graph \
$outputdir/apc-LineVoltage-$starttime-$endtime.png \
$options -t "LineVoltage" -v "VAC" \
DEF:lva=$rrdfile:LineVoltage:AVERAGE \
DEF:lvn=$rrdfile:LineVoltage:MIN \
DEF:lvm=$rrdfile:LineVoltage:MAX \
DEF:lvl=$rrdfile:LineVoltage:LAST \
VDEF:lvaa=lva,AVERAGE \
VDEF:lvnn=lvn,MINIMUM \
VDEF:lvmm=lvm,MAXIMUM \
VDEF:lvll=lvl,LAST \
TEXTALIGN:left \
LINE2:lva#0000cc:"" \
GPRINT:lvaa:"avg\: %6.2lf %s" \
GPRINT:lvnn:"min\: %6.2lf %s" \
GPRINT:lvmm:"max\: %6.2lf %s" \
GPRINT:lvll:"lst\: %6.2lf %s"

rrdtool graph \
$outputdir/apc-RunTimeRemaining-$starttime-$endtime.png \
$options -t "RunTimeRemaining" -v "minutes" \
DEF:rtra=$rrdfile:RunTimeRemaining:AVERAGE \
DEF:rtrn=$rrdfile:RunTimeRemaining:MIN \
DEF:rtrm=$rrdfile:RunTimeRemaining:MAX \
DEF:rtrl=$rrdfile:RunTimeRemaining:LAST \
CDEF:rtrad=rtra,6000,/ \
CDEF:rtrnd=rtrn,6000,/ \
CDEF:rtrmd=rtrm,6000,/ \
CDEF:rtrld=rtrl,6000,/ \
VDEF:rtraa=rtrad,AVERAGE \
VDEF:rtrnn=rtrnd,MINIMUM \
VDEF:rtrmm=rtrmd,MAXIMUM \
VDEF:rtrll=rtrld,LAST \
TEXTALIGN:left \
LINE2:rtrad#0000cc:"" \
GPRINT:rtraa:"avg\: %6.2lf %s" \
GPRINT:rtrnn:"min\: %6.2lf %s" \
GPRINT:rtrmm:"max\: %6.2lf %s" \
GPRINT:rtrll:"lst\: %6.2lf %s"
