<?php
$start_time = microtime(true);
$debug = false;
//$debug = true;

// if there is no input, return to index.php
if (!isset($_GET["search"])) return;
 
$input = $_GET["search"];

$input = trim($input);

if (empty($input))
	return;

echo "<h1>", gettext("results"), "</h1>\n";

// the hyphen must be the last in the set
if (!preg_match("/^[a-zA-Z0-9:._%-]+$/", $input)) {
	echo "<p>", gettext("unexpected characters on input"), "\n";
	return;
}


// input recognition
if (preg_match("/(_|%)/", $input)) {

	/*
	there is an wildcard in search term
	so there is no point to use regular expressions or resolve

	every select would be with "like", user is looking directly to the db

	it's possible to reduce number of searched columns
	using the search term characteristics
	*/

	debug_print("wildcard");

	if (preg_match("/([g-z]|-)/i", $input)) {

		/*
		DNS name - only one column

		don't care about other MAC address formats
		because user is searching directly in the tables
		*/

		debug_print("DNS name");

		//$condition4 = $condition6 = "dns like '$input'";
		$name_condition = "like '$input'";

	} elseif (preg_match("/:/", $input)) {

		// there is colon, so look only into columns "mac" or "ip6"

		debug_print("MAC or IPv6 address");

		//$condition4 = "mac like '$input'";
		//$condition6 = "mac like '$input' or ip6 like '$input'";
		$maca_condition = "like '$input'";
		$ip6a_condition = "like '$input'";

	} elseif (preg_match("/\./", $input)) {

		/*
		dot means an IPv4 addres or DNS name

		don't care about other MAC address formats
		because user is searching directly in the tables
		*/

		if (preg_match("/([[:alpha:]]|-)/", $input)) {

			// DNS name - only one column

			debug_print("DNS name");

			//$condition4 = $condition6 = "dns like '$input'";
			$name_condition = "like '$input'";

		} else {

			debug_print("IPv4 address or DNS name");

			//$condition4 = "ip like '$input' or dns like '$input'";
			//$condition6 = "dns like '$input'";
			$ip4a_condition = "like '$input'";
			$name_condition = "like '$input'";
		}

	} else {

		// end of fantasy, lets search everything

		debug_print("simple full search, don't expect high speed");

		//$condition4 = "mac like '$input' or ip like '$input' or dns like '$input'";
		//$condition6 = "mac like '$input' or ip6 like '$input' or dns like '$input'";
		$maca_condition = "like '$input'";
		$ip4a_condition = "like '$input'";
		$ip6a_condition = "like '$input'";
		$name_condition = "like '$input'";
	}

} else {

	// there is no wilcard on input, so prepare % for some queries

	$like_input = "%$input%";
	debug_print($like_input);

	/*
	try to get rid of "like"
	and choose the path with or without resolving
	*/
	if (preg_match("/:/", $input)) {

		/*
		colon is branch without DNS
		deal with MAC or IPv6 address here
		*/

		if (preg_match("/^([[:xdigit:]]{2}:){5}[[:xdigit:]]{2}$/", $input)) {
			debug_print("MAC address xx:xx:xx:xx:xx:xx");

			//$condition4 = $condition6 = "mac = '$input'";
			$maca_condition = "= '$input'";

		} else {

			debug_print("part of MAC, trimmed MAC, IPv6 address");

			/*
			dirty code calling for rewrite

			it is in fact unfinished code
			copied from previous recognition

			such ugly and fast expansion
			when _% must be considered
			*/

			$modified_input = preg_replace("/(:)/", "$1%", $like_input);
			debug_print($modified_input);
			//$condition4 = "mac like '$modified_input'";
			//$condition6 = "mac like '$modified_input' or ip6 like '$modified_input'";
			$maca_condition = "like '$modified_input'";
			$ip6a_condition = "like '$modified_input'";
		}
	} else {

		/*
		according the RFC 1123
		the DNS must be taken in consideration here
		*/

		debug_print("branch with the DNS burden");

		// so start with resolving and prepare appendages

		//$ip4add = $ip6add = "";

		// resolving - first plain $input, then append $domain
		debug_print("resolving");
		debug_time();

		foreach (array("$input", "$input"."$domain") as $dns) {

			/*
			here should be a support for CNAME chains and loops
			see RFC 1034
			*/

			if (checkdnsrr($dns, "CNAME")) {

				$records = dns_get_record($dns, DNS_CNAME);

				/*
				this looks odd but for shortname
				is checkdnsrr() true but returns nothing
				so the assignment will be one or none
				*/

				foreach ($records as $record) {

					echo "<p>";
					printf(gettext("The name %s points to %s (CNAME)."),
					$dns, $record["target"]);
					echo "\n";

					$dns = $record["target"];
				}
			}
			if (checkdnsrr($dns, "A")) {
				$resolved_ip4a = dns_get_record($dns, DNS_A);
				//foreach ($resolved_ip4a as $record)
				//	$ip4add .= " or ip = '$record[ip]'";
			}
			if (checkdnsrr($dns, "AAAA")) {
				$resolved_ip6a = dns_get_record($dns, DNS_AAAA);
				//foreach ($resolved_ip6a as $record) {
				//	$ip6 = expand_ipv6($record["ipv6"]);
				//	$ip6add .= " or ip6 = '$ip6'";
				//}
			}
		}
		//debug_print($ip4add);
		//debug_print($ip6add);
		debug_time();
		if (preg_match("/^([[:xdigit:]]{2}-){5}[[:xdigit:]]{2}$/", $input)) {
			debug_print("MAC address xx-xx-xx-xx-xx-xx or DNS name");
			$normalized_input = str_replace("-", ":", $input);
			//$condition4 =
			//	"mac = '$normalized_input' or dns like '$like_input'$ip4add";
			//$condition6 =
			//	"mac = '$normalized_input' or dns like '$like_input'$ip6add";
			$maca_condition = "= '$normalized_input'";
			$name_condition = "like '$like_input'";
		} elseif (preg_match("/^([[:xdigit:]]{4}-){2}[[:xdigit:]]{4}$/", $input)) {
			debug_print("MAC address xxxx-xxxx-xxxx or DNS name");
			$normalized_input = str_replace("-", "", $input);
			$normalized_input = rtrim(
					preg_replace("/([[:xdigit:]]{2})/", "$1:", $normalized_input), ":");
			//$condition4 =
			//	"mac = '$normalized_input' or dns like '$like_input'$ip4add";
			//$condition6 =
			//	"mac = '$normalized_input' or dns like '$like_input'$ip6add";
			$maca_condition = "= '$normalized_input'";
			$name_condition = "like '$like_input'";
		} elseif (preg_match("/^([[:xdigit:]]{4}\.){2}[[:xdigit:]]{4}$/", $input)) {
			debug_print("MAC address xxxx.xxxx.xxxx or DNS name");
			$normalized_input = str_replace(".", "", $input);
			$normalized_input = rtrim(
					preg_replace("/([[:xdigit:]]{2})/", "$1:", $normalized_input), ":");
			//$condition4 =
			//	"mac = '$normalized_input' or dns like '$like_input'$ip4add";
			//$condition6 =
			//	"mac = '$normalized_input' or dns like '$like_input'$ip6add";
				$maca_condition = "= '$normalized_input'";
				$name_condition = "like '$like_input'";
		} elseif
		(preg_match("/^([[:digit:]]{1,3}\.){3}[[:digit:]]{1,3}$/", $input)) {
			debug_print("IPv4 address or DNS name");
			//$condition4 = "ip like '$like_input' or dns like '$like_input'$ip4add";
			//$condition6 = "dns like '$like_input'$ip6add";
			$ip4a_condition = "like '$like_input'";
			$name_condition = "like '$like_input'";
		} elseif (preg_match("/[g-z]/i", $input)) {
			debug_print("DNS name");
			//$condition4 = "dns like '$like_input'$ip4add";
			//$condition6 = "dns like '$like_input'$ip6add";
			$name_condition = "like '$like_input'";
		} else {

			debug_print("I recognize no more, it's necessary to get through parts recognition");
			debug_print("simple full search, don't expect high speed");
			//$condition4 = "mac like '$like_input' or ip like '$like_input' or
			//	dns like '$like_input'$ip4add";
			//$condition6 = "mac like '$like_input' or ip6 like '$like_input' or
			//	dns like '$like_input'$ip6add";
			$maca_condition = "like '$like_input'";
			$ip4a_condition = "like '$like_input'";
			$ip6a_condition = "like '$like_input'";
			$name_condition = "like '$like_input'";
		}

		// end of part burdened with DNS

	}
	// end of recognition, when there is no wildcard on input
}

debug_time();

$connection = mysql_connect($dbhost, $dbuser, $dbpass);

if (!$connection)
	exit(gettext("database connection error"));

mysql_select_db($dbname);

debug_time();


$something_found = false;


// FDB

if (isset($maca_condition)) {

	$result = mysql_query("select mac, agent, port, sysName,
			mibfdbport.time collected
			from fdb2graph_leafport join snmpagent on agent = id
			join mibfdbport using(agent, mac, port)
			where mac $maca_condition order by collected desc");

	$num_rows = mysql_num_rows($result);

	if (!$num_rows) {

		/*
		* no MAC found on leaf ports
		* try all ports which haven't neighbouring switch
		*/

		$result = mysql_query("select
				mac, agent, port, sysName,
				mibfdbport.time collected
				from mibfdbport join snmpagent on agent = id
				where mac $maca_condition
				and port not in
				(select port_a from fdb2graph_neighbour
				 where agent_a = agent)
				order by collected desc");

		$num_rows = mysql_num_rows($result);
	}

	if ($num_rows) {
		$something_found = true;
		echo "<h2>", gettext("FDB table"), "</h2>\n";
		echo "<p>\n";
		echo gettext("number of records found:"), " $num_rows\n";
		echo "<table summary=\"", gettext("FDB table"), "\">\n";
		echo "<tr>",
				 "<th>", gettext("MAC address"),
				 "<th>", gettext("switch"),
				 "<th>", gettext("port"),
				 "<th>", gettext("collected"),
				 "\n";

		$row_number = 0;
		$timestamp = time();

		while ($row = mysql_fetch_array($result)) {

			$agent_name = empty($row["sysName"]) ?
				gettext("agent no.") . " " . $row["agent"]
				: $row["sysName"];

			echo "<tr class=\"monospace ",
			     tr_aging_class(++$row_number,
					     $timestamp - $row["collected"]),
			     "\">";

			echo "<td>";
			print_url("mac.php?id=" . $row["mac"], $row["mac"]);

			echo "<td>";
			print_url("snmp/mib/fdb.php?agent="
					. $row["agent"], $agent_name);

			echo "<td>";
			print_url("snmp/mib/fdbport.php?agent="
					. $row["agent"]
					. "&amp;port="
					. $row["port"], $row["port"]);

			echo "<td class=\"center\">",
			     date_formated($row["collected"]);

			echo "\n";

		}

		echo "</table>\n";
	}
}


// ARP

if (isset($maca_condition) or isset($ip4a_condition) or isset($resolved_ip4a)
		or isset($name_condition)) {
	$where = "";
	$order = "order by time desc";
	if (isset($maca_condition)) {
		$where .= "arp.mac $maca_condition";
	}
	if (isset($ip4a_condition)) {
		if (! empty($where)) $where .= " or ";
		$where .= "arp.ip $ip4a_condition";
	}
	if (isset($resolved_ip4a)) {
		foreach ($resolved_ip4a as $record) {
			if (! empty($where)) $where .= " or ";
			$where .= "arp.ip = '$record[ip]'";
		}
	}
	if (! empty($where)) {
		debug_print($where);
		$query_arp = "select mac, ip, time from arp
			where $where";
	}
	if (isset($name_condition)) {
		$query_rr4 = "select mac, ip, arp.time time
			from arp left join rr4 on arp.ip = rr4.address
			where rr4.name $name_condition";
		if (! empty($query_arp))
			$query = "($query_arp) union ($query_rr4) $order";
		else
			$query = "$query_rr4 $order";
	} else
		$query = "$query_arp $order";
	debug_print($query);
	$result = mysql_query($query);
	$num_rows = mysql_num_rows($result);
	if ($num_rows > 0) {
		$something_found = true;
		echo "<h2>", gettext("ARP table"), "</h2>\n";
		echo "<p>\n";
		echo gettext("number of records found:"), " $num_rows\n";
		echo "<table summary=\"", gettext("ARP table"), "\">\n";
		echo "<tr>",
				 "<th>", gettext("MAC address"),
				 "<th>", gettext("IPv4 address"),
				 "<th>", gettext("last seen"),
				 "<th>", gettext("DNS name"),
				 "\n";

		$row_number = 0;
		$timestamp = time();

		while ($row = mysql_fetch_array($result)) {
			echo "<tr class=\"monospace ",
			     tr_aging_class(++$row_number,
					     $timestamp - $row["time"]),
			     "\">",
			     "<td><a href=\"mac.php?id=", $row["mac"],
			     "\">",
			     $row["mac"], "</a>",
			     "<td><a href=\"ipv4.php?id=", $row["ip"],
			     "\">",
			     $row["ip"], "</a>",
			     "<td class=\"center\">",
			     date_formated($row["time"]),
			     "<td>", try_addr2name($row["ip"]),
			     "\n";
		}
		echo "</table>\n";
	}
}

debug_time();


// ND

if (isset($maca_condition) or isset($ip6a_condition) or isset($resolved_ip6a)
		or isset($name_condition)) {
	$where = "";
	$order = "order by time desc";
	if (isset($maca_condition)) {
		$where .= "nd.mac $maca_condition";
	}
	if (isset($ip6a_condition)) {
		if (! empty($where)) $where .= " or ";
		$where .= "nd.ip6 $ip6a_condition";
	}
	if (isset($resolved_ip6a)) {
		foreach ($resolved_ip6a as $record) {
			$ip6a = expand_ipv6($record["ipv6"]);
			if (! empty($where)) $where .= " or ";
			$where .= "nd.ip6 = '$ip6a'";
		}
	}
	if (! empty($where)) {
		debug_print($where);
		$query_nd = "select mac, ip6, time from nd
			where $where";
	}
	if (isset($name_condition)) {
		$query_rr6 = "select mac, ip6, nd.time time
			from nd left join rr6 on nd.ip6 = rr6.address
			where rr6.name $name_condition";
		if (! empty($query_nd))
			$query = "($query_nd) union ($query_rr6) $order";
		else
			$query = "$query_rr6 $order";
	} else
		$query = "$query_nd $order";
	debug_print($query);
	$result = mysql_query($query);
	$num_rows = mysql_num_rows($result);
	if ($num_rows > 0) {
		$something_found = true;
		echo "<h2>", gettext("ND table"), "</h2>\n";
		echo "<p>\n";
		echo gettext("number of records found:"), " $num_rows\n";
		echo "<table summary=\"", gettext("ND table"), "\">\n";
		echo "<tr>",
				 "<th>", gettext("MAC address"),
				 "<th>", gettext("IPv6 address"),
				 "<th>", gettext("last seen"),
				 "<th>", gettext("DNS name"),
				 "\n";

		$row_number = 0;
		$timestamp = time();

		while ($row = mysql_fetch_array($result)) {

			echo "<tr class=\"monospace ",
			     tr_aging_class(++$row_number,
					     $timestamp - $row["time"]),
			     "\">";

			echo "<td>";
			print_url("mac.php?id=" . $row["mac"], $row["mac"]);

			echo "<td>";
			print_url("ipv6.php?id=" . $row["ip6"],
					ipv6_compress($row["ip6"]));

			echo "<td class=\"center\">",
			     date_formated($row["time"]);

			echo "<td>", try_addr2name($row["ip6"]);

			echo "\n";

		}
		echo "</table>\n";
	}
}


// SNMP
if (isset($like_input))
	// no wildcards on input
	$snmp_input = $like_input;
else
	// a wildcard on input
	$snmp_input = $input;
	
$query = "select id, time, sysDescr, sysObjectID, sysName,
	dot1dTpFdbEntry, ifXEntry
	from snmpagent
	where sysName like '$snmp_input' or sysDescr like '$snmp_input'
	order by time desc";

$result = mysql_query($query);
$num_rows = mysql_num_rows($result);
if ($num_rows > 0) {
	$something_found = true;
	echo "<h2>SNMP</h2>\n";
	echo "<p>\n";
	echo gettext("number of records found:"), " $num_rows\n";
	echo "<table summary=\"SNMP\">\n";
	echo "<tr>",
	     "<th>time",
	     "<th>sysName",
	     "<th>sysDescr",
	     "<th>links",
	     "\n";

	$row_number = 0;
	$timestamp = time();

	while ($row = mysql_fetch_assoc($result)) {
		echo "<tr class=\"monospace ",
		     tr_aging_class(++$row_number, $timestamp - $row["time"]),
		     "\">";
		echo "<td class=\"center\"><nobr>",
		     date_formated($row["time"]), "</nobr>";
		echo "<td><nobr><a href=\"snmp/agent.php?id=", $row["id"],
		     "\">", $row["sysName"], "</a></nobr>";
		echo "<td>", $row["sysDescr"];
		echo "<td><nobr><a href=\"snmp/mib/ifgraph.php?agent=",
		     $row["id"], "\">if graphs</a>&nbsp;",
		     "&nbsp;<a href=\"snmp/mib/iftable.php?agent=",
		     $row["id"], "\">ifTable</a>&nbsp;";
		if ($row["dot1dTpFdbEntry"] == "y")
			echo "&nbsp;<a href=\"snmp/mib/fdb.php?agent=",
			$row["id"], "\">FDB</a>&nbsp;";
		if ($row["ifXEntry"] == "y")
			echo "&nbsp;<a href=\"snmp/mib/ifxtable.php?agent=",
			     $row["id"], "\">ifXTable</a>&nbsp;";
		 if ($organization = sysObjectID2organization(
					 $row["sysObjectID"]))
			 echo "&nbsp;<a href=\"snmp/mib/",
			      $organization, ".php",
			      "?agent=", $row["id"],
			      "\">$organization</a>";
		echo "</nobr>\n";
	}
	echo "</table>\n";
}


// switches

if (isset($maca_condition) or isset($ip4a_condition) or isset($resolved_ip4a)
		or isset($name_condition)) {
	$filter = "";
	if (isset($maca_condition)) {
		$filter .= "checked.mac $maca_condition";
	}
	if (isset($ip4a_condition)) {
		if (! empty($filter)) $filter .= " or ";
		$filter .= "arp.ip $ip4a_condition";
	}
	if (isset($resolved_ip4a)) {
		foreach ($resolved_ip4a as $record) {
			if (! empty($filter)) $filter .= " or ";
			$filter .= "arp.ip = '$record[ip]'";
		}
	}
	if (isset($name_condition)) {
		if (! empty($filter)) $filter .= " or ";
		$filter .= "name $name_condition";
	}
	debug_print($filter);
	$query = "select
		mac, ip, lastsnmpread, dns id
		from checked left join arp using (mac)
		left join rr4 on arp.ip = rr4.address
		where dns != '' and hasfdb = 'y'
		and arp.time =
		(select max(sub.time) from arp sub where sub.mac = arp.mac)
		and ($filter) order by lastsnmpread desc";
	debug_print($query);
	$result = mysql_query($query);
	$num_rows = mysql_num_rows($result);
	if ($num_rows > 0) {
		$something_found = true;
		echo "<h2>", gettext("table of switches"), "</h2>\n";

		echo "<p>";
		echo gettext("This section is considered obsolete and will be removed.");
		echo "\n";

		echo "<p>";
		echo gettext("The neighborhood of switches is available via FDB link at every switch in the SNMP section above.");
		echo "\n";

		echo "<p>";
		echo gettext("Let me know if you wish continue to use the table below.");
		echo "\n";

		echo "<p>\n";
		echo gettext("number of records found:"), " $num_rows\n";
		echo "<table summary=\"", gettext("table of switches"), "\">\n";
		echo "<tr>",
				 "<th>", gettext("MAC address"),
				 "<th>", gettext("IPv4 address"),
				 "<th>", gettext("DNS name"),
				 "<th>", gettext("last read via SNMP"),
				 "<th>", gettext("neighborhood"),
				 "\n";

		$row_number = 0;
		$timestamp = time();

		while ($row = mysql_fetch_array($result)) {
			echo "<tr class=\"monospace ",
			     tr_aging_class(++$row_number,
					     $timestamp - $row["lastsnmpread"]),
			     "\">",
			     "<td><a href=\"mac.php?id=", $row["mac"],
			     "\">",
			     $row["mac"], "</a>",
			     "<td><a href=\"ipv4.php?id=", $row["ip"],
			     "\">",
			     $row["ip"], "</a>",
			     "<td>", try_addr2name($row["ip"]),
			     "<td class=\"center\">",
			     date_formated($row["lastsnmpread"]),
			     "<td><a href=\"switch.php?id=", $row["id"],
			     "\">",
			     $row["id"], "</a>",
			     "\n";
		}
		echo "</table>\n";
	}
}


// DNS

// PHP 5 >= 5.2.0
//if (filter_var($input, FILTER_VALIDATE_IP)) {

// disable gethostbyaddr() warning until filter_var can be used
error_reporting(E_ALL ^ E_WARNING);
$resolved_name = address2name($input);
error_reporting(E_ALL);

if ($resolved_name != "" or isset($resolved_ip4a) or isset($resolved_ip6a)) {
	$something_found = true;

	echo "<h2>DNS</h2>\n";
	echo "<p>\n";
	echo "<table summary=\"DNS\">\n";
	echo "<tr>",
	     "<th>", gettext("name"),
	     "<th>", gettext("address"),
	     "\n";

	if ($resolved_name != "") {

		echo "<tr>";
		echo "<td>";
		print_url("dns.php?id=$resolved_name", $resolved_name);
		echo "<td>";
		if (preg_match("/\./", $input))
			print_url("ipv4.php?id=$input", $input);
		else {
			$ipv6_address = ipv6_compress($input);
			print_url("ipv6.php?id=$ipv6_address", $ipv6_address);
		}
		echo "\n";

	} else {

		if (isset($resolved_ip4a)) {

			foreach ($resolved_ip4a as $record) {

				echo "<tr>";
				echo "<td>";
				print_url("dns.php?id=$record[host]",
						$record["host"]);
				echo "<td>";
				print_url("ipv4.php?id=$record[ip]",
						$record["ip"]);
				echo "\n";
			}
		}

		if (isset($resolved_ip6a)) {

			foreach ($resolved_ip6a as $record) {

				echo "<tr>";
				echo "<td>";
				print_url("dns.php?id=$record[host]",
						$record["host"]);
				echo "<td>";
				$ipv6_address = ipv6_compress($record["ipv6"]);
				print_url("ipv6.php?id=$ipv6_address",
						$ipv6_address);
				echo "\n";
			}
		}
	}

	echo "</table>\n";
}


if (! $something_found) echo "<p>", gettext("nothing found"), "\n";

debug_time();

?>
